// simple.frag

#extension GL_EXT_gpu_shader4 : enable

uniform sampler2D textureImage;
uniform int textureMode;
uniform int fogMode;
uniform int       shadowVolumeModeCartoon;
uniform float     boldnessCartoon;
varying float s;
varying float t; 

void FogEffect(){

    float z = gl_FragCoord.z/ gl_FragCoord.w;
	
    vec4 fogColor    = gl_Fog.color;
    fogColor[3] = gl_FragColor[3];
    float fogDensity = gl_Fog.density;

    float fogFactor = 0.0;
    if (fogMode == 3){
	fogFactor = (gl_Fog.end - z) / (gl_Fog.end - gl_Fog.start);
    }
    else if (fogMode == 2){
	float LOG2 = 1.442695;
	fogFactor = exp2( -gl_Fog.density * z * LOG2 );
    }
    else {
	float LOG2 = 1.442695;
	fogFactor = exp2( -gl_Fog.density * gl_Fog.density * z * z * LOG2 );
    }
    
    fogFactor = clamp(fogFactor, 0.0, 1.0);
 
    gl_FragColor = mix(fogColor, gl_FragColor, fogFactor);
}

void main (void)
{
    vec4 texColor = texture2D(textureImage, gl_TexCoord[0].st);
    if (textureMode == 1){
	texColor[0] += 0.0001;
	texColor[1] += 0.0001;
	texColor[2] += 0.0001;
    }
    else{
	texColor[0] = 1;
	texColor[1] = 1;
	texColor[2] = 1;
	texColor[3] = 1;
    }
    
    float alpha = gl_FrontLightProduct[0].ambient[3];
    if (alpha > 1.0) alpha = 1.0;
        
    vec4 ambient = gl_FrontMaterial.ambient * float(shadowVolumeModeCartoon);

    if (t < boldnessCartoon/*0.25*/){
	gl_FragColor[0] = 0.0;
	gl_FragColor[1] = 0.0;
	gl_FragColor[2] = 0.0;
    }
    else{
	if (s < 0.2){
	    gl_FragColor = ambient;
	}
	else if (s < 0.75){
	    gl_FragColor = ambient +
		gl_FrontMaterial.diffuse;
	}
	else{
	    gl_FragColor = ambient + 
		gl_FrontMaterial.diffuse + 
		gl_FrontMaterial.specular;
	}
    }
    gl_FragColor *= texColor;
    
    gl_FragColor[3] = alpha;

    if (fogMode != 0) FogEffect();
}
